! =================================================================================================================================
! MODULE       : stomate_kill
!
! CONTACT      : orchidee-help _at_ ipsl.jussieu.fr
!
! LICENCE      : IPSL (2006)
! This software is governed by the CeCILL licence see ORCHIDEE/ORCHIDEE_CeCILL.LIC
!
!>\BRIEF       Simulate mortality of individuals and update biomass, litter and 
!! stand density of the PFT
!!
!!\n DESCRIPTION : Simulate mortality of individuals and update biomass, litter and 
!! stand density of the PFT. This module replaces lpj_gap/kill.  Biomass actually
!! dies here and is moved to the litter pools.  This does not work with the DGVM
!! at the moment.
!!
!! RECENT CHANGE(S): None
!!
!! REFERENCE(S) : 
!! - Sitch, S., B. Smith, et al. (2003), Evaluation of ecosystem dynamics,
!!         plant geography and terrestrial carbon cycling in the LPJ dynamic 
!!         global vegetation model, Global Change Biology, 9, 161-185.\n
!! - Waring, R. H. (1983). "Estimating forest growth and efficiency in relation 
!!         to canopy leaf area." Advances in Ecological Research 13: 327-354.\n
!! 
!! SVN          :
!! $HeadURL: svn://forge.ipsl.jussieu.fr/orchidee/branches/ORCHIDEE-DOFOCO/ORCHIDEE/src_stomate/stomate_kill.f90 $
!! $Date: 2013-09-27 09:59:24 +0200 (Fri, 27 Sep 2013) $
!! $Revision: 1485 $
!! \n
!_ ================================================================================================================================

MODULE stomate_kill

  ! modules used:

  USE stomate_data
  USE pft_parameters
  USE constantes
  USE ioipsl_para 
  USE stomate_prescribe
  USE stomate_mark_kill,      ONLY : distribute_mortality_biomass
  USE function_library,       ONLY : nmax, wood_to_dia, check_surface_area, &
                                     check_mass_balance, sort_circ_class_biomass, &
                                     cc_to_biomass
  USE sapiens_lcchange,       ONLY : merge_biomass_pfts

  IMPLICIT NONE

  ! private & public routines

  PRIVATE
  PUBLIC natural_mortality, natural_mortality_clear, mortality_clean
  
  ! Variable declaration 

  LOGICAL, SAVE             :: firstcall = .TRUE.     !! first call flag
!$OMP THREADPRIVATE(firstcall)

CONTAINS


!! ================================================================================================================================
!! SUBROUTINE   : natural_mortality_clear
!!
!>\BRIEF        Set the firstcall flag back to .TRUE. to prepare for the next simulation.
!_ ================================================================================================================================
  
  SUBROUTINE natural_mortality_clear
     firstcall = .TRUE.
  END SUBROUTINE natural_mortality_clear


!! ================================================================================================================================
!! SUBROUTINE   : natural_mortality
!!
!>\BRIEF        Transfer dead biomass to litter and update stand density for trees
!!              which die of natural causes.
!!
!! DESCRIPTION  : This routine has a simple purpose: kill individuals by natural causes.  
!!   The variable circ_class_kill indicates how many individuals need to be killed by the various
!!   processes in the code, and is calculated in other modules. natural_mortality takes
!!   this variable and decides on the correct order of which to actually kill the
!!   trees, making sure that we don't double count.  
!!
!! RECENT CHANGE(S): 
!!
!! MAIN OUTPUT VARIABLE(S): ::circ_class_biomass; ::circ_class_n density of individuals, 
!! ::mortality mortality (fraction of trees that is dying per time step)
!!
!! REFERENCE(S)   :
!! - Sitch, S., B. Smith, et al. (2003), Evaluation of ecosystem dynamics,
!!         plant geography and terrestrial carbon cycling in the LPJ dynamic 
!!         global vegetation model, Global Change Biology, 9, 161-185.
!! - Waring, R. H. (1983). "Estimating forest growth and efficiency in relation 
!!         to canopy leaf area." Advances in Ecological Research 13: 327-354.
!!
!! FLOWCHART    : None
!!\n
!_ ================================================================================================================================
 
  SUBROUTINE natural_mortality (npts,  bm_to_litter, &
       circ_class_biomass, circ_class_kill, circ_class_n, &
       veget_cov_max, biomass_cut)

    !! 0. Variable and parameter declaration

    !! 0.1 Input variables 
    INTEGER(i_std), INTENT(in)                                   :: npts               !! Domain size (-)
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in)                   :: veget_cov_max          !! Maximum fraction of vegetation type 

    !! 0.2 Output variables

    !! 0.3 Modified variables   
    REAL(r_std), DIMENSION(npts,nvm,ncirc,nparts,nelements), &                         
                                              INTENT(inout)      :: circ_class_biomass !! Biomass of the components of the model  
                                                                                       !! tree within a circumference
                                                                                       !! class @tex $(gC ind^{-1})$ @endtex  
    REAL(r_std), DIMENSION(npts,nvm,ncirc), INTENT(inout)        :: circ_class_n       !! Number of individuals in each circ class
                                                                                       !! @tex $(m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,ncirc,nfm_types,ncut_times),&
                                                  INTENT(inout)  :: circ_class_kill    !! Number of trees within a circ that needs
                                                                                       !! to be killed @tex $(ind m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nparts,nelements), &
                                                  INTENT(inout)  :: bm_to_litter       !! Biomass transfer to litter 
                                                                                       !! @tex $(gC m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nparts,nelements,nfm_types,ncut_times), &   
                                                  INTENT(inout)  :: biomass_cut        !! Biomass change due to  ncut

    !! 0.4 Local variables
    INTEGER(i_std)                                               :: ipts, ivm, ipar    !! Indices
    INTEGER(i_std)                                               :: iele, icir, imbc   !! Indices
    INTEGER(i_std)                                               :: ifm,icut           !! Indices
    INTEGER,DIMENSION(nvm)                                       :: nkilled            !! the number of grid points at which a given
                                                                                       !! given PFT's biomass has been reduced to zero
    REAL(r_std), DIMENSION(npts,nvm,nmbcomp,nelements)           :: check_intern       !! Contains the components of the internal
                                                                                       !! mass balance chech for this routine
                                                                                       !! @tex $(gC pixel^{-1} dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nelements)                   :: closure_intern     !! Check closure of internal mass balance
                                                                                       !! @tex $(gC pixel^{-1} dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nelements)                   :: pool_start         !! Start pool of this routine 
                                                                                       !! @tex $(gC pixel^{-1} dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nelements)                   :: pool_end           !! End pool of this routine 
                                                                                       !! @tex $(gC pixel^{-1} dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm)                             :: veget_cov_max_begin    !! temporary storage of veget_cov_max to check area conservation
    REAL(r_std), DIMENSION(npts,nvm,nparts,nelements)            :: init_biomass       !! Temporary array of the initial biomass
    REAL(r_std), DIMENSION(nparts,nelements)                     :: actu_biomass       !! Temporary array of the actual biomass

!_ ================================================================================================================================

 !! Initialize variables

    !! 1.1 Set firstcall flag
    IF ( firstcall ) THEN
       firstcall = .FALSE.
    ENDIF

    IF (printlev.GE.3) WRITE(numout,*) 'Entering gap. Use constant mortality = ', &
         ok_constant_mortality

    !! 1.2 Initialize variables
    nkilled(:)=0
    
    !! 1.1 Initialize local printlev
    !printlev_loc=get_printlev('natural_mortality')

    ! 1.3 Initialize check for mass balance closure
    IF (err_act.GT.1) THEN

       check_intern(:,:,:,:) = zero
       pool_start(:,:,:) = zero
       DO ipar = 1,nparts
          DO iele = 1,nelements        
             !  Initial litter and biomass
             DO icir = 1,ncirc
                pool_start(:,:,iele) = pool_start(:,:,iele) + &
                     (circ_class_biomass(:,:,icir,ipar,iele) * &
                     circ_class_n(:,:,icir) * veget_cov_max(:,:))
                     
             ENDDO
             pool_start(:,:,iele) = pool_start(:,:,iele) + &
                     bm_to_litter(:,:,ipar,iele) * veget_cov_max(:,:)
          ENDDO
       ENDDO

       !! 1.3 Initialize check for area conservation
       veget_cov_max_begin(:,:) = veget_cov_max(:,:)
       
    ENDIF ! err_act.GT.1

    ! The total biomass at the start of the routine
    init_biomass = cc_to_biomass(npts,nvm,circ_class_biomass(:,:,:,:,:),&
         circ_class_n(:,:,:))


 !! 3. Kill plants

    DO  ipts = 1,npts ! loop over land_points

       DO ivm = 2,nvm  ! loop over #PFT

          IF(veget_cov_max(ipts,ivm) == zero)THEN
              ! this vegetation type is not present, so no reason to do the 
              ! calculation
               CYCLE
          ENDIF

          !! First, all the plants that are supposed to be killed by fire are killed.
          !! We do not yet know how to handle this, so this will have to be taken
          !! into account when SPITFIRE is coupled.
!!$          DO iele = 1,nelements

             !! All of the natural death is grouped 
             !! together in one pool since all of the biomass will be left on 
             !! site (moved to the litter pools).
             DO ifm=1,nfm_types

                DO icut=1,ncut_times

                   ! Since we gave circ_class_kill the same dimensions as the
                   ! harvest pool, we need to explicitly say which combinations
                   ! of ifm and icut are killed in which ways.  Currently, I am
                   ! putting all natural death into ifm_none, even if it happens
                   ! in another management type (for example, self-thinning will 
                   ! happen with ifm_thin, but it's really a natural death).
                   IF(ifm .NE. ifm_none)CYCLE
                   IF((icut .NE. icut_thin) .AND. &
                        (icut .NE. icut_clear) .AND. &
                        (icut .NE. icut_storm_break) .AND. &
                        (icut .NE. icut_storm_uproot)) CYCLE

                   ! Killing is done a little differently for trees and 
                   ! non-trees, since circ_classes are not defined for non-trees.
                   IF(is_tree(ivm))THEN

                      DO icir=1,ncirc

                         ! Debug
                         IF(test_pft == ivm .AND. test_grid == ipts .AND. &
                              printlev_loc>=4) THEN
                            WRITE(numout,*) 'killing before: ',ipts,ivm,icir,ifm,icut,&
                                 circ_class_kill(ipts,ivm,icir,ifm,icut),&
                                 circ_class_n(ipts,ivm,icir)
                         ENDIF
                         !

                         ! move the dead biomass to the respective litter pool
                         bm_to_litter(ipts,ivm,:,:) = bm_to_litter(ipts,ivm,:,:) + &
                              circ_class_biomass(ipts,ivm,icir,:,:)*&
                              circ_class_kill(ipts,ivm,icir,ifm,icut)
                         
                         ! remove the number of individuals that died
                         circ_class_n(ipts,ivm,icir) = circ_class_n(ipts,ivm,icir) - &
                              circ_class_kill(ipts,ivm,icir,ifm,icut)

                         ! Here, it's possible that the number of individuals left
                         ! will be a very, very small amount.  If it's very low,
                         ! we will just move all that biomass to the dead pool
                         ! and set the number of individuals equal to zero,
                         ! effectively killing the circ class.
                         IF(circ_class_n(ipts,ivm,icir) .LE. min_stomate)THEN

                            bm_to_litter(ipts,ivm,:,:) = bm_to_litter(ipts,ivm,:,:) + &
                                 circ_class_biomass(ipts,ivm,icir,:,:)*&
                                 circ_class_n(ipts,ivm,icir)

                            ! zero the number of individuals in this circ class
                            circ_class_n(ipts,ivm,icir) = circ_class_n(ipts,ivm,icir) - &
                                 circ_class_n(ipts,ivm,icir)

                            ! If there are no individuals left then the biomass
                            ! in that circ should be set to zero. If not some of 
                            ! the IF-loops will fail because circ_class_n = 0 and
                            ! circ_class_biomass = 0 is used as a logic test later
                            ! in the code
                            circ_class_biomass(ipts,ivm,icir,:,:) = zero
                            
                         ENDIF

                         ! Debug
                         IF(test_pft == ivm .AND. test_grid == ipts .AND. &
                              printlev_loc>=4) THEN
                            WRITE(numout,*) 'killing after: ',ipts,ivm,icir,ifm,icut,&
                                 circ_class_kill(ipts,ivm,icir,ifm,icut),&
                                 circ_class_n(ipts,ivm,icir)
                         ENDIF
                         !-

                     ENDDO ! loop over circ classes

                     ! Actual biomass after mortality has been accounted for
                     actu_biomass = cc_to_biomass(npts,nvm,&
                          circ_class_biomass(ipts,ivm,:,:,:),&
                          circ_class_n(ipts,ivm,:))

                     ! Calculate the biomass change due to ncuts
                     biomass_cut(ipts,ivm,:,:,ifm,icut) = &
                          biomass_cut(ipts,ivm,:,:,ifm,icut) + &
                          init_biomass(ipts,ivm,:,:) - actu_biomass(:,:)

                     ! Update the initial biomass to avoid double counting
                     init_biomass(ipts,ivm,:,:) = actu_biomass(:,:)
                          

                   ELSE ! Grasses and croplands

                      !! WARNING !!
                      ! This is not very clean.  We don't use circ classes for grasses 
                      ! and crops.  All mortality is done based on biomass.  However, 
                      ! it's cleaner here to just pass circ_class_kill from 
                      ! stomate_mark_kill, 
                      ! and it is in line with what is done for the forests.  So we take
                      ! the total grass/crop biomass that is scheduled for killing in 
                      ! stomate_mark_kill and define circ_class_kill(:,:,inatural,1) and 
                      ! circ_class_biomass(:,:,1,:,:) so that the product of these two
                      ! matches the amount of biomass scheduled for killing.  We do NOT
                      ! change the number of individuals after killing, either.

                      ! Let us compute the fraction of plants which die of natural
                      ! causes in this step.
                      ! Individuals are not killed for grasses and crops, so we cannot
                      ! calculate the mortality as a function of the individuals

                      ! We renormalize by circ_class_n below.  If it's zero,
                      ! we have an error.  Of course, if it's zero there is no
                      ! reason to be here in the first place.
                      IF(circ_class_n(ipts,ivm,1) .LE. min_stomate) THEN
                         CYCLE
                      ENDIF

                      ! move the dead biomass to the respective litter pool
                      bm_to_litter(ipts,ivm,:,:) = bm_to_litter(ipts,ivm,:,:) + &
                           circ_class_biomass(ipts,ivm,1,:,:)*&
                           circ_class_kill(ipts,ivm,1,ifm,icut)

                      ! circ_class_biomass has to change such that the biomass
                      ! and circ_class_biomass*circ_class_n are in sync.  So we
                      ! will just sync it here.  This has to be done this way
                      ! since circ_class_n never changes for grass/crops.
                      ! Below we combine the calculation of the new biomass
                      ! biomass = circ_class_biomass * circ_class_n
                      ! biomass_killed = circ_class_biomass * circ_class_kill
                      ! Given that we keep circ_class_n constant for grass and crops
                      ! we recalculate circ_class_biomass as 
                      ! (biomass-biomass_killed)/circ_class_n
                      circ_class_biomass(ipts,ivm,1,:,:) = &
                           circ_class_biomass(ipts,ivm,1,:,:) * &
                           (circ_class_n(ipts,ivm,1) - &
                           circ_class_kill(ipts,ivm,1,ifm,icut)) / &
                           circ_class_n(ipts,ivm,1)

                      ! In order for something to be prescribed, ind must be zero.
                      ! We don't change ind and circ_class_n if we just kill some
                      ! of the biomass, but if we kill all of it we must set
                      ! it to zero in order for it to be prescribed.
                      ! IF(ivm == test_pft) WRITE(numout,*) 'Killing? ', &
                      ! SUM(biomass(ipts,ivm,:,iele))
                      IF(SUM(SUM(circ_class_biomass(ipts,ivm,:,:,icarbon),1),1) .LE. &
                           min_stomate)THEN

                         ! Move the remaining biomass into the proper
                         ! litter pools
                         DO ipar = 1,nparts
                            DO iele = 1,nelements
                               bm_to_litter(ipts,ivm,ipar,iele) = &
                                    bm_to_litter(ipts,ivm,ipar,iele) + &
                                    SUM(circ_class_biomass(ipts,ivm,:,ipar,iele) * &
                                    circ_class_n(ipts,ivm,:))
                            ENDDO
                         ENDDO

                         ! Set the biomass and the number of individuals to zero
                         circ_class_biomass(ipts,ivm,:,:,:) = zero
                         circ_class_n(ipts,ivm,:) = zero                         

                      ENDIF
                      
                   ENDIF ! checking for a tree

                   ! This PFT should never be killed again.  
                   ! To make sure of that, reset all the
                   ! variables to zero.
                   circ_class_kill(ipts,ivm,:,ifm,icut) = zero

                ENDDO ! reason of mortality

             ENDDO ! management types

!!$          ENDDO ! loop over elements

       ENDDO  ! loop over pfts

    END DO ! loop over land points

 !! 4. Check numerical consistency of this routine

    IF (err_act.GT.1) THEN


       ! 4.2 Check surface area
       CALL check_surface_area("stomate_natural_mortality", npts, &
            veget_cov_max_begin, veget_cov_max,'pft')

       ! 4.3 Mass balance closure 
       pool_end = zero
       DO ipar = 1,nparts
          DO iele = 1,nelements
             DO icir = 1,ncirc
                pool_end(:,:,iele) = pool_end(:,:,iele) + &
                     (circ_class_biomass(:,:,icir,ipar,iele) * &
                     circ_class_n(:,:,icir) * veget_cov_max(:,:))
             ENDDO
             pool_end(:,:,iele) = pool_end(:,:,iele) + &
                  bm_to_litter(:,:,ipar,iele) * veget_cov_max(:,:)
          ENDDO
       ENDDO

       ! 4.3.2 Calculate mass balance
       ! Common processes
       DO iele=1,nelements
          check_intern(:,:,ipoolchange,iele) = -un * (pool_end(:,:,iele) - &
               pool_start(:,:,iele)) 
       ENDDO

       closure_intern(:,:,:) = zero
       DO imbc = 1,nmbcomp
          DO iele=1,nelements
             ! Debug
             IF (printlev_loc>=4) WRITE(numout,*) &
                  'check_intern, ivm, imbc, iele, ', imbc, &
                  iele, check_intern(:,test_pft,imbc,iele)
             !-
             closure_intern(:,:,iele) = closure_intern(:,:,iele) + &
                  check_intern(:,:,imbc,iele)
          ENDDO
       ENDDO

       ! 4.3.3 Check mass balance closure
       CALL check_mass_balance("stomate_natural_mortality", closure_intern, &
            npts, pool_end, pool_start, veget_cov_max, 'pft')
      
    ENDIF ! err_act.GT.1

    IF (printlev.GE.4) WRITE(numout,*) 'Leaving natural_mortality'

  END SUBROUTINE natural_mortality


!! ================================================================================================================================
!! SUBROUTINE   : mortality_clean
!!
!>\BRIEF        After biomass has been killed, there are some clean-up operations
!!              to do.
!!
!! DESCRIPTION  : If all the biomass has been killed for a PFT, we want to reset
!!                some counters.  There is also the chance that we have
!!                killed all the biomass in a circ class of trees.  If this is
!!                the case, we want to redistribut ethe biomass in the remaining
!!                classes so that all circ classes have some biomass in them.
!!                A circ class with no biomass causes allocation to crash.
!!
!! RECENT CHANGE(S): 
!!
!! MAIN OUTPUT VARIABLE(S): ::circ_class_biomass
!!
!! REFERENCE(S)   :
!!
!! FLOWCHART    : None
!!\n
!_ ================================================================================================================================
 
  SUBROUTINE mortality_clean (npts, circ_class_biomass, &
       circ_class_n, age, everywhere, leaf_age, &
       leaf_frac, plant_status, when_growthinit, circ_class_dist, &
       age_stand, PFTpresent, last_cut, mai_count, &
       veget_cov_max, npp_longterm, KF, atm_to_bm, &
       wstress_season, lm_lastyearmax, lignin_struc, lignin_wood, &
       som, litter, dt_days, &
       forest_managed, bm_to_litter, lab_fac, &
       gpp_daily, resp_maint, &
       resp_growth, npp_daily, use_reserve, mai, pai, &
       rue_longterm, previous_wood_volume, species_change_map,&
       tau_eff_leaf, tau_eff_sap, tau_eff_root, moiavail_growingseason,&
       k_latosa_adapt, lpft_replant, cn_leaf_min_season, &
       nstress_season, soil_n_min, n_uptake_daily, p_O2, bact,&
       CN_som_litter_longterm,matrixA,matrixV,vectorB,vectorU)

    !! 0. Variable and parameter declaration

    !! 0.1 Input variables 
    INTEGER(i_std), INTENT(in)                       :: npts                   !! Domain size (-)
    REAL(r_std), DIMENSION(ncirc), INTENT(in)        :: circ_class_dist        !! The probability distribution of trees 
                                                                               !! in a circ class in case of a 
                                                                               !! redistribution (unitless).
    REAL(r_std), DIMENSION(:,:), INTENT(in)          :: tau_eff_root           !! Effective root turnover time that accounts
                                                                               !! waterstress (days)
    REAL(r_std), DIMENSION(:,:), INTENT(in)          :: tau_eff_sap            !! Effective sapwood turnover time that accounts
                                                                               !! waterstress (days)
    REAL(r_std), DIMENSION(:,:), INTENT(in)          :: tau_eff_leaf           !! Effective leaf turnover time that accounts
                                                                               !! waterstress (days)  
    REAL(r_std), INTENT(in)                          :: dt_days                !! Time step of vegetation dynamics for stomate (days)
    INTEGER(i_std), DIMENSION (:,:), INTENT(in)      :: forest_managed         !! forest management flag
     LOGICAL, DIMENSION(npts,nvm), INTENT(in)        :: lpft_replant           !! Set to true if a PFT has been clearcut
                                                                               !! and needs to be replaced by another species
    INTEGER(i_std), DIMENSION (:,:), INTENT(in)       :: species_change_map    !! A map which gives the PFT number that each


    !! 0.2 Output variables

    !! 0.3 Modified variables   
    REAL(r_std), DIMENSION(:,:,:,:,:), INTENT(inout) :: circ_class_biomass     !! Biomass of the components of the model  
                                                                               !! tree within a circumference
                                                                               !! class @tex $(gC ind^{-1})$ @endtex  
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: plant_status           !! Growth and phenological status of the plant 
                                                                               !! see constants voor values
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: age                    !! Mean age (years)  
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: when_growthinit        !! How many days ago was the beginning of the 
                                                                               !! growing season (days)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: everywhere             !! Is the PFT everywhere in the grid box or very
                                                                               !! localized (after its introduction)
    REAL(r_std), DIMENSION(:,:,:), INTENT(inout)     :: leaf_age               !! Leaf age (days)
    REAL(r_std), DIMENSION(:,:,:), INTENT(inout)     :: leaf_frac              !! Fraction of leaves in leaf age class 
                                                                               !! (unitless;0-1)
    INTEGER(i_std), DIMENSION(:,:), INTENT(inout)    :: age_stand              !! Age of the forest stand (years)  
    INTEGER(i_std), DIMENSION(:,:), INTENT(inout)    :: last_cut               !! Years since last thinning (years)
    INTEGER(i_std), DIMENSION(:,:), INTENT(inout)    :: mai_count              !! The number of times we've
                                                                               !! calculated the volume increment
                                                                               !! for a stand
    LOGICAL, DIMENSION(:,:), INTENT(inout)           :: PFTpresent             !! PFT present (0 or 1)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: KF                     !! Scaling factor to convert sapwood mass into leaf 
                                                                               !! mass (m)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: k_latosa_adapt         !! Leaf to sapwood area adapted for long 
                                                                               !! term water stress (m)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: veget_cov_max              !! "maximal" coverage fraction of a PFT on the ground
                                                                               !! (unitless, 0-1)
    REAL(r_std), DIMENSION(:,:,:), INTENT(inout)     :: atm_to_bm              !! C and N taken from the atmosphere to get C to create  
                                                                               !! the seedlings @tex (gC.m^{-2}dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: wstress_season         !! Water stress factor, based on hum_rel_daily
                                                                               !! (unitless, 0-1) 

    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: npp_longterm           !! "long term" net primary productivity
                                                                               !! @tex ($gC m^{-2} year^{-1}$) @endtex
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: lm_lastyearmax         !! last year's maximum leaf mass for each PFT 
                                                                               !! @tex ($gC m^{-2}$) @endtex
    REAL(r_std), DIMENSION(:,:,:), INTENT(inout)     :: lignin_struc           !! ratio Lignine/Carbon in structural litter,
                                                                               !! above and below ground
    REAL(r_std), DIMENSION(:,:,:), INTENT(inout)     :: lignin_wood            !! ratio Lignine/Carbon in woody litter,
                                                                               !! above and below ground
    REAL(r_std), DIMENSION(:,:,:,:), INTENT(inout)   :: som                    !! carbon pool: active, slow, or passive 
                                                                               !! @tex ($gC m^{-2}$) @endtex 
    REAL(r_std), DIMENSION(:,:,:,:,:), INTENT(inout) :: litter                 !! metabolic and structural litter, above and 
                                                                               !! below ground @tex ($gC m^{-2}$) @endtex 
    REAL(r_std), DIMENSION(:,:,:), INTENT(inout)     :: circ_class_n           !! Number of individuals in each circ class
                                                                               !! @tex $(ind m^{-2})$ @endtex  
    REAL(r_std), DIMENSION(:,:,:,:), INTENT(inout)   :: bm_to_litter           !! Transfer of biomass to litter 
                                                                               !! @tex $(gC m^{-2} dtslow^{-1})$ @endtex 
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: lab_fac                !! Activity of labile pool factor (??units??)
!!$    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: gdd_from_growthinit    !! growing degree days, since growthinit 
!!$                                                                               !! for crops
!!$    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: gdd_midwinter          !! Growing degree days (K), since midwinter 
!!$                                                                               !! (for phenology) - this is written to the
!!$                                                                               !!  history files 
!!$    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: time_hum_min           !! Time elapsed since strongest moisture 
!!$                                                                               !! availability (days)
!!$    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)  :: hum_min_dormance       !! minimum moisture during dormance 
!!$                                                                               !! (0-1, unitless)
!!$    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: gdd_m5_dormance        !! Growing degree days (K), threshold -5 deg 
!!$                                                                               !! C (for phenology)
!!$    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: ncd_dormance           !! Number of chilling days (days), since 
!!$                                                                               !! leaves were lost (for phenology) 
!!$    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: moiavail_month         !! "Monthly" moisture availability (0 to 1, 
!!$                                                                               !! unitless) 
!!$    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: moiavail_week          !! "Weekly" moisture availability 
!!$                                                                               !! (0 to 1, unitless)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: gpp_daily              !! Daily gross primary productivity  
                                                                               !! @tex $(gC m^{-2} dtslow^{-1})$ @endtex
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: resp_maint             !! Maintenance respiration  
                                                                               !! @tex $(gC m^{-2} dtslow^{-1})$ @endtex 
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: resp_growth            !! Growth respiration  
                                                                               !! @tex $(gC m^{-2} dtslow^{-1})$ @endtex
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: npp_daily              !! Net primary productivity 
                                                                               !! @tex $(gC m^{-2} dtslow^{-1})$ @endtex
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: use_reserve            !! Mass taken from carbohydrate reserve 
                                                                               !! @tex $(gC m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)  :: mai                    !! The mean annual increment
                                                                               !! @tex $(m**3 / m**2 / year)$ @endtex 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)  :: pai                    !! The period annual increment
                                                                               !! @tex $(m**3 / m**2 / year)$ @endtex          
!!$    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: ngd_minus5             !! Number of growing days (days), threshold 
!!$                                                                               !! -5 deg C (for phenology)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: rue_longterm           !! Longterm radiation use efficiency 
                                                                               !! (??units??) 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)  :: previous_wood_volume   !! The volume of the tree trunks
                                                                               !! in a stand for the previous year.
                                                                               !! @tex $(m**3 / m**2 )$ @endtex
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: moiavail_growingseason !! Mean growingseason moisture 
                                                                               !! availability (0 to 1, unitless)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: cn_leaf_min_season     !! Seasonal min CN ratio of leaves 
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: nstress_season         !! N-related seasonal stress (used for allocation) 
    REAL(r_std), DIMENSION(:,:,:), INTENT(inout)     :: soil_n_min             !! mineral nitrogen in the soil (gN/m**2)  
                                                                               !! (first index=npts, second index=nvm, third index=nnspec) 
    REAL(r_std), DIMENSION(:,:,:), INTENT(inout)     :: n_uptake_daily         !! Daily N uptake by plants
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: p_O2                   !! partial pressure of oxigen in the soil (hPa)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: bact                   !! denitrifier biomass (gC/m**2)
    REAL(r_std), DIMENSION(:,:,:),INTENT(inout)      :: CN_som_litter_longterm !! Longterm CN ratio of litter and som pools (gC/gN)
    REAL(r_std), DIMENSION(:,:,:,:), INTENT(inout)   :: matrixA                !! Matrix containing the fluxes between the carbon
                                                                               !! pools per sechiba time step @tex $(gC.m^2.day^{-1})$
                                                                               !! @endtex
    REAL(r_std), DIMENSION(:,:,:), INTENT(inout)     :: vectorB                !! Vector containing the litter increase per sechiba
                                                                               !! time step @tex $(gCm^{-2})$ @endtex
    REAL(r_std), DIMENSION(:,:,:,:), INTENT(inout)   :: matrixV                !! Matrix containing the accumulated values of matrixA
    REAL(r_std), DIMENSION(:,:,:), INTENT(inout)     :: vectorU                !! Matrix containing the accumulated values of VectorB

    !! 0.4 Local variables
    REAL(r_std)                                      :: share_young            !! Share of the veget_cov_max of the youngest age class
                                                                               !! (unitless, 0-1)
    INTEGER(i_std)                                   :: ipts,ivm,ilage         !! Indices
    INTEGER(i_std)                                   :: icir,jcir,ilev         !! Indices
    INTEGER(i_std)                                   :: ipar,iele,imbc         !! Indices
    INTEGER(i_std)                                   :: ilit,icarb             !! Indices
    INTEGER(i_std)                                   :: inc, ispec             !! Indices
    LOGICAL                                          :: lredistribute          !! Flag if we need to redistribute individuals
                                                                               !! among the circ classes
    INTEGER,DIMENSION(nvm)                           :: nkilled                !! the number of grid points at which a given
                                                                               !! given PFT's biomass has been reduced to 
                                                                               !! zero
    REAL(r_std), DIMENSION(npts,nvm,nmbcomp,nelements)&
                                                     :: check_intern           !! Contains the components of the internal
                                                                               !! mass balance chech for this routine
                                                                               !! @tex $(gC pixel^{-1} dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nelements)       :: closure_intern         !! Check closure of internal mass balance
                                                                               !! @tex $(gC pixel^{-1} dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nelements)       :: pool_start             !! Start pool of this routine 
                                                                               !! @tex $(gC pixel^{-1} dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nelements)       :: pool_end               !! End pool of this routine 
                                                                               !! @tex $(gC pixel^{-1} dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(ncirc)                    :: old_trees_left
    REAL(r_std), DIMENSION(ncirc)                    :: circ_class_n_new
    REAL(r_std), DIMENSION(ncirc,nparts,nelements)   :: circ_class_biomass_new
    REAL(r_std)                                      :: trees_needed
    LOGICAL                                          :: lyoungest
    REAL(r_std), DIMENSION(npts,nvm,nleafages)       :: new_leaf_frac          !! Temporary variable for leaf_frac (unitless;0-1)
    REAL(r_std), DIMENSION(ncirc)                    :: est_circ_class_n       !! Temporary variable for circ_class_n of the 
                                                                               !! established vegetation @tex $(ind m^{-2})$ @endtex
    REAL(r_std), DIMENSION(ncirc)                    :: tmp_circ_class_n       !! Temporary variable for circ_class_n of the 
                                                                               !! established vegetation @tex $(ind m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,ncirc)           :: new_circ_class_n       !! Temporary variable for circ_class_n of the
                                                                               !! new vegetation
                                                                               !! @tex $(ind m^{-2})$ @endtex
    REAL(r_std), DIMENSION(ncirc,nparts,nelements)   :: est_circ_class_biomass !! Temporary variable for circ_class_biomass of the
                                                                               !! established vegetation @tex $(g C ind^{-1})$ @endtex
    REAL(r_std), DIMENSION(ncirc,nparts,nelements)   :: tmp_circ_class_biomass !! Temporary variable for circ_class_biomass of the
                                                                               !! established vegetation @tex $(g C ind^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,ncirc,nparts,nelements) &
                                                     :: new_circ_class_biomass !! Temporary variable for circ_class_biomass of the
                                                                               !! new vegetation @tex $(g C ind^{-1})$ @endtex
    REAL(r_std), DIMENSION(ncirc)                    :: est_circ_class_dia     !! Variable to store temporary values for 
                                                                               !! circ_class (m)
    REAL(r_std), DIMENSION(ncirc)                    :: tmp_circ_class_dia     !! Variable to store temporary values for 
                                                                               !! circ_class (m)
    REAL(r_std), DIMENSION(npts,nvm,ncirc)           :: new_circ               !! Temporary variable for circ (m)
    REAL(r_std), DIMENSION(npts,nvm,nelements)       :: new_atm_to_bm          !! Temporary variable for atm_to_bm 
                                                                               !! @tex (gC.m^{-2}dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm)                 :: new_age                !! Temporary variable for age (years)
    REAL(r_std), DIMENSION(npts,nvm)                 :: new_lm_lastyearmax     !! Temporary variable for lm_last_year 
                                                                               !! @tex ($gC m^{-2}$) @endtex
    REAL(r_std), DIMENSION(npts,nvm)                 :: new_everywhere         !! Temporary variable for everywhere (unitless, 0-1)
    REAL(r_std), DIMENSION(npts,nvm)                 :: dum_when_growthinit    !! Dummy for when_growthinit (days)
    REAL(r_std), DIMENSION(npts,nvm)                 :: dum_npp_longterm       !! Dummy for npp_long_term
                                                                               !! @tex ($gC m^{-2} year^{-1}$) @endtex
    INTEGER(i_std)                                   :: ivma
    LOGICAL, DIMENSION(npts,nvm)                     :: dum_PFTpresent         !! Dummy for PFTpresent (0 or 1)
    REAL(r_std), DIMENSION(npts,nvm)                 :: dum_plant_status       !! Dummy for plant_status (true/false)
    REAL(r_std)                                      :: moi_bank               !! bank to store moiavail_growingseason of the cleared
                                                                               !! PFT's (unitless; 0-1)
    REAL(r_std), DIMENSION(npts,nvm)                 :: loss_gain              !! The same as delta_veget but distributed of all
                                                                               !! age classes and thus taking the age-classes into 
                                                                               !! account (unitless, 0-1)
    REAL(r_std)                                      :: total_losses           !! Sum of losses only in delta_veget (unitless, 0-1)
    REAL(r_std), DIMENSION(nlitt,nlevs,nelements)    :: litter_bank            !! Bank to store the litter that becomes available when
                                                                               !! part of a PFT is removed @tex ($gC m^{-2}$) @endtex
    REAL(r_std), DIMENSION(ncarb)                    :: soil_bank              !! Bank to store soil carbon that becomes available when
                                                                               !! part of a PFT is removed  @tex ($gC m^{-2}$) @endtex
    REAL(r_std), DIMENSION(nlevs)                    :: struct_ltr_bank        !! Bank to store the lignin in structural litter that 
                                                                               !! becomes available when part of a PFT is removed 
                                                                               !! (0-1,unitless)
    REAL(r_std),DIMENSION(nlevs)                     :: woody_ltr_bank         !! Bank to store the lignin in woody litter that 
                                                                               !! becomes available when part of a PFT is removed 
                                                                               !! (0-1,unitless)
    REAL(r_std),DIMENSION(nvm,nparts,nelements)      :: fresh_litter           !! Pool of fresh litter that is being released during 
                                                                               !! LCC @tex ($gC m^{-2}$) @endtex 
    REAL(r_std),DIMENSION(nparts,nelements)          :: fresh_ltr_bank         !! Bank to store all the fresh litter from site 
                                                                               !! clearing during LCC. Weighted value of fresh_litter
                                                                               !! @tex ($gC m^{-2}$) @endtex
    INTEGER(i_std)                                   :: iyoung                 !! index of the youngest age class of that species
    REAL(r_std), DIMENSION(npts,nvm,norphans,nelements) &
                                               :: orphan_flux_local            !! Storage for fluxes of PFTs that no longer exist. 
                                                                               !! This is only for co2bm at the moment, since that
                                                                               !! is the only one used in this routine. Following the 
                                                                               !! total destruction of a PFT by death
                                                                               !! (veget_cov_max_new = 0), a flux from before death needs  
                                                                               !! storage @tex $(gC m^{-2} dtslow^{-1})$ @endtex
    REAL(r_std), DIMENSION(nlitt,nlevs)                :: litter_weight_young  !! The fraction of litter on the young
                                                                               !! PFT.
                                                                               !! @tex $-$ @endtex
    REAL(r_std), DIMENSION(npts,nvm)                   :: veget_cov_max_begin      !! Temporairy variable to check area conservation 
    REAL(r_std), DIMENSION(npts,nvm,nlevels_tot)       :: dummy1               !! Dummy variable for prescribe
    LOGICAL                                            :: dummy2               !! Dummy variable for prescribe 
!_ ================================================================================================================================

    IF (printlev.GE.3) WRITE(numout,*) 'Entering mortality_clean.'

 !! 1. Initialize check for mass balance closure
  
    !! 1.1 Initialize local printlev
    !printlev_loc=get_printlev('mortality_clean')

    !! 1.2 Initialize check for mass balance closure 
    IF (err_act.GT.1) THEN

       check_intern(:,:,:,:) = zero
       pool_start(:,:,:) = zero
       DO iele = 1,nelements
          DO ipar = 1,nparts
             DO icir = 1,ncirc
                !  Initial biomass
                pool_start(:,:,iele) = pool_start(:,:,iele) + &
                     (circ_class_biomass(:,:,icir,ipar,iele) * &
                     circ_class_n(:,:,icir) * veget_cov_max(:,:)) 
             ENDDO
          ENDDO

          ! The litter and soil carbon pools can be moved around
          ! if age classes are changed.
       
          ! Litter pool (gC m-2) *  (m2 m-2) 
          DO ilit = 1,nlitt
             DO ilev = 1,nlevs
                pool_start(:,:,iele) = pool_start(:,:,iele) + &
                     litter(:,ilit,:,ilev,iele) * veget_cov_max(:,:)
             ENDDO
          ENDDO

          ! Soil carbon (gC m-2) *  (m2 m-2)
          DO icarb = 1,ncarb
             pool_start(:,:,iele) = pool_start(:,:,iele) + &
                  som(:,icarb,:,iele) * veget_cov_max(:,:)
          ENDDO

          check_intern(:,:,iatm2land,iele) = -un * &
               atm_to_bm(:,:,iele) * veget_cov_max(:,:)

       ENDDO

       ! Denitrifying bacteria (only C)
       pool_start(:,:,icarbon) = pool_start(:,:,icarbon) + &
            bact(:,:) * veget_cov_max(:,:)

       ! Nitrogen only
       DO ispec = 1,nnspec
          pool_start(:,:,initrogen) = pool_start(:,:,initrogen) + &
               soil_n_min(:,:,ispec) * veget_cov_max(:,:)
       ENDDO

       ! Specific fluxes
       check_intern(:,:,iatm2land,icarbon) = &
            check_intern(:,:,iatm2land,icarbon) - un *&
            gpp_daily(:,:) * veget_cov_max(:,:)
       check_intern(:,:,iland2atm,icarbon) = &
            check_intern(:,:,iland2atm,icarbon) + &
            (resp_maint(:,:) + resp_growth(:,:)) * &
            veget_cov_max(:,:)

       !! 1.3 Initialize check for area conservation
       veget_cov_max_begin(:,:) = veget_cov_max(:,:)

    ENDIF ! err_act.GT.1


 !! 2. Redistribute biomass
  
    DO  ipts = 1,npts ! loop over land_points

       DO ivm = 2,nvm  ! loop over #PFT

          IF(veget_cov_max(ipts,ivm) == zero)THEN
             ! this vegetation type is not present, so no reason to do the 
             ! calculation.
             CYCLE
          ENDIF

          ! First we check to see if one of the circ classes is empty.  
          ! We only need to do this if there is some biomass, since we 
          ! don't care if all of the circ classes are empty.
          IF(SUM(SUM(circ_class_biomass(ipts,ivm,:,:,icarbon),1)) .GT. min_stomate)THEN
             
             IF(is_tree(ivm))THEN

                ! By setting this to .TRUE. redistribution
                ! will be taken care of every day. Initially this flag was 
                ! only true at the end of the year but that resulted in 
                ! spikes and pulses.
                lredistribute=.TRUE.

                IF(lredistribute) THEN

                   ! Debug
                   IF(printlev_loc>=4 .AND. ivm==test_pft)THEN
                      WRITE(numout,*) 'Start of redistributing biomass in mortality_clean'
                      WRITE(numout,*) 'ipts,ivm',ipts,ivm
                      WRITE(numout,*) 'circ_class_n, ',circ_class_n(ipts,ivm,:)
                      WRITE(numout,*) 'circ_class_biomass, ', &
                           SUM(circ_class_biomass(ipts,ivm,:,:,icarbon),2)
                   ENDIF
                   !-

                   ! What we want to do is recreate the circumference class distribution,
                   ! since if we have hit this point we have at least one class that
                   ! is empty and this will cause the allocation to crash.  

                   ! The first step is to decide of the distribution of individuals 
                   ! that we want in each class, for example, a uniform or exponential 
                   ! distribution. This was made this an input parameter. Note
                   ! that this is a decisison with very high consequences. The diamter
                   ! range may vary but the distribution is always the same.

                   ! Now we need to populate the new distribution of trees among
                   ! the circumference classes, and move the heartwood and sap masses 
                   ! to the new distributions. We also need to move the other pools.  
                   ! This is tricky because the allometric relations for the new
                   ! tree sizes will NOT give the same amount of biomass for
                   ! the non-woody pools.  Let us first try it just distributing
                   ! everything equally, and then if that doesn't work we can
                   ! redistribute the non-woody pools more cleverly, even if that 
                   ! means we change the total amount of biomass we have in this
                   ! stand.  I want to try it this way because we conserve biomass 
                   ! this way, and I hope that the stress on the trees will be
                   ! small enough that it doesn't cause problems.  This
                   ! redistribution should only happen rarely, so the trees
                   ! should have a chance to equilibrate.
                   circ_class_n_new(:)=circ_class_dist(:)*SUM(circ_class_n(ipts,ivm,:))
                   circ_class_biomass_new(:,:,:)=zero

                   ! now we populate the new classes
                   old_trees_left(:)=circ_class_n(ipts,ivm,:)

                   ! The subsequent calculation nicely closes the carbon
                   ! balance but within the precision 10-16. If there are
                   ! enough trees (first case in the IF-loop), we introduce
                   ! a precision error in the number of trees. This implies
                   ! that when the number of trees is very small, the
                   ! precision issue can become a numerical issue. This is 
                   ! especially true for the largest circumference classes 
                   ! because they contain the fewest individuals. 
                   ! We observed the largest circumference classes 
                   ! containing 10-4 to 10-5 gC less than the previous 
                   ! class. The rest of the code shouldn't care too much 
                   ! whether the circumferences classes are in order or not 
                   ! but to reduce the chances this problem occurs
                   ! we inverted the DO-loops. That way the precision error
                   ! is carried to the smaller diameter classes 
                   ! which have more individuals and so the precision error
                   ! stays a precision issue. Note that this routine is
                   ! called every day so the problem is likely to be 
                   ! corrected the next day.
                   DO icir=ncirc,1,-1

                      trees_needed=circ_class_n_new(icir)

                      DO jcir=ncirc,1,-1

                         IF(trees_needed .LE. zero)EXIT

                         ! Debug
                         IF(printlev_loc>=4 .AND. ivm==test_pft)THEN
                            WRITE(numout,*) 'start of loop'
                            WRITE(numout,*) 'circ_class_biomass_new, C',&
                                 icir,SUM(circ_class_biomass_new(icir,:,icarbon))
                             WRITE(numout,*) 'circ_class_biomass_new, N',&
                                 icir,SUM(circ_class_biomass_new(icir,:,initrogen))
                            WRITE(numout,*) 'old_trees_left, ', &
                                 jcir, old_trees_left(jcir)
                            WRITE(numout,*) 'trees needed, ',trees_needed
                         ENDIF
                         !-

                         IF(old_trees_left(jcir) .GE. trees_needed )THEN

                            ! we can get all the trees we need from this class
                            ! and don't have to continue searching
                            circ_class_biomass_new(icir,:,:)=circ_class_biomass_new(icir,:,:)+&
                                 circ_class_biomass(ipts,ivm,jcir,:,:)*trees_needed
                            old_trees_left(jcir)=old_trees_left(jcir)-trees_needed
                            trees_needed = zero

                            ! Debug
                            IF(printlev_loc>=4 .AND. ivm==test_pft)THEN
                               WRITE(numout,*) 'enough trees'
                            ENDIF
                            !-
   
                            EXIT

                         ELSE

                            ! the trees in this class are not sufficient, so we
                            ! need to move all of them to the new class.
                            circ_class_biomass_new(icir,:,:)=circ_class_biomass_new(icir,:,:)+&
                                 circ_class_biomass(ipts,ivm,jcir,:,:)*&
                                 old_trees_left(jcir)
                            trees_needed = trees_needed-old_trees_left(jcir)
                            old_trees_left(jcir)=zero

                            ! Debug
                            IF(printlev_loc>=4 .AND. ivm==test_pft)THEN
                               WRITE(numout,*) 'not enough trees'
                            ENDIF
                            !-

                         ENDIF
                         
                         ! Debug
                         IF(printlev_loc>=4 .AND. ivm==test_pft)THEN
                            WRITE(numout,*) 'start of loop'
                            WRITE(numout,*) 'circ_class_biomass_new, C',&
                                 icir,SUM(circ_class_biomass_new(icir,:,icarbon))
                             WRITE(numout,*) 'circ_class_biomass_new, N',&
                                 icir,SUM(circ_class_biomass_new(icir,:,initrogen))
                            WRITE(numout,*) 'old_trees_left, ', &
                                 jcir, old_trees_left(jcir)
                            WRITE(numout,*) 'trees needed, ',trees_needed
                         ENDIF
                         !-

                      ENDDO ! jcir=1,ncirc

                   ENDDO ! icir=1,ncirc

                   ! right now, circ_class_biomass_new gives the total biomass
                   ! in each class, but it should only be for a model tree. 
                   ! So let's normalize it.
                   DO icir=1,ncirc
                      circ_class_biomass_new(icir,:,:) = &
                           circ_class_biomass_new(icir,:,:)/&
                           circ_class_n_new(icir)
                   ENDDO

                   ! Check whether the order was preserved. We only want to
                   ! preserve the order for the carbon biomass. Nitrogen follows
                   ! carbon. We expect that the biomass of an individual tree 
                   ! increases with an increasing circ class. This is probably 
                   ! not important for the correct functioning of the model. So,
                   ! if this turns out to be computationally expensive it is 
                   ! worth trying without. We try to preserve the order as
                   ! an additional mean to control the quality of the simulations.
                   ! A strict order also helps to interpret the output. Hence, 
                   ! if the order was not preserved we will sort the biomasses. 
                   ! Note that this subroutine first checks whether the order 
                   ! was preseverd.
                   CALL sort_circ_class_biomass(circ_class_biomass_new,&
                           circ_class_n_new)

                   ! Now update the variables that pass this information around
                   DO icir=1,ncirc
                      circ_class_biomass(ipts,ivm,icir,:,:) = &
                           circ_class_biomass_new(icir,:,:)
                      circ_class_n(ipts,ivm,icir) = circ_class_n_new(icir)
                   ENDDO

                ENDIF ! redistribute
                
                ! Debug
                IF(printlev_loc>=4 .AND. ivm==test_pft)THEN
                   WRITE(numout,*) 'End of redistributing biomass in mortality_clean'
                   WRITE(numout,*) 'ipts,ivm',ipts,ivm
                   WRITE(numout,*) 'circ_class_n, ',circ_class_n(ipts,ivm,:)
                   WRITE(numout,*) 'circ_class_biomass - C, ', &
                        SUM(circ_class_biomass(ipts,ivm,:,:,icarbon),2)
                    WRITE(numout,*) 'circ_class_biomass - N, ', &
                        SUM(circ_class_biomass(ipts,ivm,:,:,initrogen),2)
                ENDIF
                !-

             ELSE

                ! we have biomass here but it's not a tree.  Don't need to do anything.

             ENDIF

          ELSEIF (SUM(SUM(circ_class_biomass(ipts,ivm,:,:,icarbon),1),1) .LT. &
               min_stomate .AND. &
               plant_status(ipts,ivm) .EQ. idead) THEN 

             ! All the biomass was killed for this site.  Some flags to
             ! be reset in this case. It is OK to kill the PFT even if 
             ! we are using species change but we can not replant yet
             ! because we want to replant with a different species this is
             ! basically the same as a land cover change and so it is best 
             ! taken care of with the sapiens_lcchange code. To end up
             ! here we need a veget_cov_max for this PFT. Another check is 
             ! through using ::PFTpresent.
             IF(PFTpresent(ipts,ivm))THEN
                
                nkilled(ivm)=nkilled(ivm)+1 ! purely an informational variable

                !! 1.5 Reinitialize vegetation characteristics in STOMATE
                plant_status(ipts,ivm) = iprescribe
                age(ipts,ivm) = zero
                when_growthinit(ipts,ivm) = large_value

                ! Specific variables for forest stands
                IF(is_tree(ivm))THEN

                   last_cut(ipts,ivm) = zero
                   mai_count(ipts,ivm) = zero
                   age_stand(ipts,ivm) = zero

                ENDIF

                !! 1.6 Update leaf ages
                DO ilage = 1, nleafages
                   
                   leaf_age(ipts,ivm,ilage) = zero 
                   leaf_frac(ipts,ivm,ilage) = zero 
                   
                ENDDO

                ! This used to be done in lpj_kill.  If we have grasses,
                ! we reset the long term GPP.  This value as 500 in the
                ! old code.  We externalized the variable, but we are
                ! leaving the default at 500.  Many PFTs should have
                ! a long term value of less than 500, so someone could
                ! do some work on this in the future.
                IF (.NOT. is_tree(ivm) .AND. .NOT. ok_constant_mortality) THEN
                   
                   npp_longterm(ipts,ivm) = npp_reset_value(ivm)

                ENDIF
              
             ENDIF ! is PFT present?

!          ELSE
!
!             CALl ipslerr_p (3,'mortality_clean',&
!                  'unexpected condition in if-loop','','')
!
          ENDIF ! check if biomass is equal to zero

          ! If we have no vegetation left, this stand has been completely cleared.
          ! We have to reset the age, regardless if it was for natural or human reasons.
          IF(is_tree(ivm))THEN
             
             ! This is essentially the same code that is found in land cover change,
             ! since the same process happens there.  Notice that here we do not
             ! need to use the _bank variables, since we assume that if a stand dies
             ! due to natural causes it will always be replaced by the same species.
             ! We also assume that if a stand is clearcut, the model will replace it
             ! by the same species.  To do so otherwise would require something more
             ! like a DGVM. That is what is being done in the species change code.
             ! If the species change code is used, the PFT will be replanted at the
             ! end of the year. Not in the middle of the year as being done here.

             ! If we are using age classes, we need to reset a lot of counters
             ! and change veget_cov_max, moving veget_cov_max from this age class to
             ! the lowest age class.  If veget_cov_max is greater than zero and there
             ! is no biomass, prescribe will attempt to grow trees here in the
             ! next timestep.  This is fine if we have no age classes, but it doesn't
             ! make any sense to prescribe trees that are 50 years old.  We should
             ! only prescribe trees which are 0 years old.
             IF( SUM(SUM(circ_class_biomass(ipts,ivm,:,:,icarbon),1),1) .LT. &
                  min_stomate .AND. (agec_group(ivm)==species_change_map(ipts,ivm) .OR. &
                  .NOT.lpft_replant(ipts,ivm))) THEN 

                ! Debug
                IF(printlev_loc>=4)THEN
                   WRITE(numout,*) 'Getting reset in mortality_clean'
                   WRITE(numout,*) 'ivm,ipts',ivm,ipts
                ENDIF
                !-
                
                IF(nagec .GT. 1)THEN

                   ! First, we need to find the youngest age class of this PFT.
                   iyoung=start_index(agec_group(ivm))
                   IF(ivm == iyoung)THEN
                      lyoungest=.TRUE.
                   ELSE
                      lyoungest=.FALSE.
                   ENDIF

                   IF(lyoungest)THEN

                      ! We don't need to do anything.  Things will be handled properly with
                      ! prescribe in the next step.

                   ELSE

                      ! All of our counters for this stand are zero.  We need to merge that
                      ! with the youngest age class.  There are a couple possibilities.
                      ! One is that nothing exists right now in the youngest age class, in 
                      ! which case the veget_cov_max of the old age class becomes that of the 
                      ! youngest and we prescribe.  Another possibility is that something 
                      ! does exist in the youngest age class, in which case we prescribe 
                      ! to the current age class and then merge biomass with the youngest.  
                      ! A third is that both the youngest age class and the current age 
                      ! class have died.  In that case we can do the same thing as if there 
                      ! is no vegetation in the young age class, we just have to make sure 
                      ! to add the veget_cov_max and not replace it.
                      ! Veget_max after PFT expansion. As ::veget_cov_max is used in the 
                      ! IF-statements we won't update it until the end of this routine. If 
                      ! we would updatebefore that, the same PFT may be subject to 
                      ! conflicting actions.
                      share_young = veget_cov_max(ipts,iyoung) / &
                           ( veget_cov_max(ipts,iyoung) + veget_cov_max(ipts,ivm) )

                      ! We also need a scaling factor which includes the litter
                      DO ilev=1,nlevs
                         DO ilit=1,nlitt
                            IF(litter(ipts,ilit,iyoung,ilev,icarbon) .GT. min_stomate)THEN

                               litter_weight_young(ilit,ilev)=&
                                    litter(ipts,ilit,iyoung,ilev,icarbon)*&
                                    veget_cov_max(ipts,iyoung)/ &
                                    (litter(ipts,ilit,ivm,ilev,icarbon)*veget_cov_max(ipts,ivm) + &
                                    litter(ipts,ilit,iyoung,ilev,icarbon)*&
                                    veget_cov_max(ipts,iyoung))

                            ELSE

                               litter_weight_young(ilit,ilev)=zero

                            ENDIF
                         END DO
                      ENDDO

                      IF( SUM(SUM(circ_class_biomass(ipts,iyoung,:,:,icarbon),1),1) .LT. &
                           min_stomate)THEN

                         ! Debug
                         IF(printlev_loc>=4)THEN
                            WRITE(numout,*) 'Merging our biomass to the youngest age class.'
                            WRITE(numout,*) 'No current biomass in the youngest age class.'
                            WRITE(numout,*) 'ipts,iyoung,ivm: ',ipts,iyoung,ivm
                            WRITE(numout,*) 'share_young: ',share_young
                            WRITE(numout,*) 'veget_cov_max(young and current): ',&
                                 veget_cov_max(ipts,iyoung),veget_cov_max(ipts,ivm)
                         ENDIF
                         !-

                         ! Is there anything in the smallest age class?  It
                         ! does not matter if the youngest age class died in this
                         ! step or not.
                         plant_status(ipts,iyoung) = iprescribe
                         plant_status(ipts,ivm) = idead
   
                         veget_cov_max(ipts,iyoung) = veget_cov_max(ipts,iyoung)+veget_cov_max(ipts,ivm)
                         
                         moiavail_growingseason(ipts,iyoung) = &
                              share_young * moiavail_growingseason(ipts,iyoung) + &
                              moiavail_growingseason(ipts,ivm) * &
                              (un - share_young)

                         ! To keep the mass balance closed we have to move the
                         ! gpp and autothropic respiration fluxes to youngest
                         ! age class
                         gpp_daily(ipts,iyoung) = share_young * gpp_daily(ipts,iyoung) + &
                              (un-share_young) * gpp_daily(ipts,ivm)
                         resp_maint(ipts,iyoung) = share_young * resp_maint(ipts,iyoung) + &
                              (un-share_young) * resp_maint(ipts,ivm)
                         resp_growth(ipts,iyoung) = share_young * resp_growth(ipts,iyoung) + &
                              (un-share_young) * resp_growth(ipts,ivm)
                         atm_to_bm(ipts,iyoung,:) = share_young * atm_to_bm(ipts,iyoung,:) + &
                              (un-share_young) * atm_to_bm(ipts,ivm,:)

                         ! I am not sure why we merge wstress here.  wstress is recalculated
                         ! everyday from moiavail_growingseason.
                         wstress_season(ipts,iyoung) = &
                              share_young * wstress_season(ipts,iyoung) + &
                              wstress_season(ipts,ivm) * (un - share_young)
                         nstress_season(ipts,iyoung) = &
                              share_young * nstress_season(ipts,iyoung) + &
                              nstress_season(ipts,ivm) * (un - share_young)
                         
                         ! The litter variables also need to be merged, since these will not
                         ! get updated in prescribe in the next step.
                         litter(ipts,:,iyoung,:,:) =   &
                              share_young * litter(ipts,:,iyoung,:,:) + &
                              litter(ipts,:,ivm,:,:) * &
                              (un - share_young)
                         som(ipts,:,iyoung,:) =  &
                              share_young * som(ipts,:,iyoung,:) + &
                              som(ipts,:,ivm,:) * &
                              (un - share_young)

                         ! We also need to merge plant N uptake
                         n_uptake_daily(ipts,iyoung,iammonium) = share_young * &
                              n_uptake_daily(ipts,iyoung,iammonium) + &
                              n_uptake_daily(ipts,ivm,iammonium) * (un - share_young)
                         n_uptake_daily(ipts,iyoung,initrate) = share_young * &
                              n_uptake_daily(ipts,iyoung,initrate) + &
                              n_uptake_daily(ipts,ivm,initrate) * (un - share_young)

                         DO ilev=1,nlevs
                            lignin_struc(ipts,iyoung,ilev) = &
                                 litter_weight_young(istructural,ilev) * &
                                 lignin_struc(ipts,iyoung,ilev) + &
                                 (un - litter_weight_young(istructural,ilev)) * &
                                 lignin_struc(ipts,ivm,ilev) 
                            lignin_wood(ipts,iyoung,ilev) = &
                                 litter_weight_young(iwoody,ilev) * &
                                 lignin_wood(ipts,iyoung,ilev) + &
                                 (un - litter_weight_young(iwoody,ilev) ) * &
                                 lignin_wood(ipts,ivm,ilev) 
                         ENDDO

                         bm_to_litter(ipts,iyoung,:,:) = &
                              share_young * bm_to_litter(ipts,iyoung,:,:) + &
                              bm_to_litter(ipts,ivm,:,:) * &
                              (un - share_young)

                         ! Update the soil pools
                         soil_n_min(ipts,iyoung,:) = &
                              share_young * soil_n_min(ipts,iyoung,:) + &
                              soil_n_min(ipts,ivm,:) * &
                              (un - share_young)
                         p_O2(ipts,iyoung) = &
                              share_young * p_O2(ipts,iyoung) + &
                              p_O2(ipts,ivm) * (un - share_young)
                         bact(ipts,iyoung) = &
                              share_young * bact(ipts,iyoung) + &
                              bact(ipts,ivm) * (un - share_young)

                         ! Leaf properties
                         cn_leaf_min_season(ipts,iyoung) = &
                              share_young * cn_leaf_min_season(ipts,iyoung) + &
                              cn_leaf_min_season(ipts,ivm) * (un - share_young)
                         ! For the analytic spinup, we need longterm CN ratios
                         ! calculated over the spinup period. If the tree grows
                         ! and move to an older age class, this is taken care of
                         ! by age_class_distr, but if PFT is killed we need to
                         ! move CN back to the first age class as well.
                         ! Otherwise, we will not get a CN ratio for soil and
                         ! litter pools the corresponds to the spinup period. 
                         IF(spinup_analytic)THEN
                            CN_som_litter_longterm(ipts,iyoung,:)=&
                              share_young * CN_som_litter_longterm(ipts,iyoung,:) + &
                              CN_som_litter_longterm(ipts,ivm,:) * (un - share_young)
                            matrixA(ipts,iyoung,:,:)=&
                              share_young *matrixA(ipts,iyoung,:,:) + &
                              matrixA(ipts,ivm,:,:) * (un -share_young)
                            matrixV(ipts,iyoung,:,:)=&
                              share_young *matrixV(ipts,iyoung,:,:) + &
                              matrixV(ipts,ivm,:,:) * (un -share_young)
                            vectorB(ipts,iyoung,:)=&
                              share_young *vectorB(ipts,iyoung,:) + &
                              vectorB(ipts,ivm,:) * (un -share_young)
                            vectorU(ipts,iyoung,:)=&
                              share_young *vectorB(ipts,iyoung,:) + &
                              vectorB(ipts,ivm,:) * (un -share_young)

                         ENDIF 

                      ELSE

                         ! There are two important things to do here.  First,
                         ! we need to prescribe biomass to this PFT, since it
                         ! is currently empty.  Then we need to merge this biomass
                         ! with what is already in the youngest age class of this PFT.

                         ! We want to make use of prescribe but it should 
                         ! be noted that the youngest PFT already contains biomass. 
                         ! Therefore we will create a temporary PFT without biomass which 
                         ! can be used to establish the  new vegetation within the 
                         ! youngest. For most of the INTENT(inout)  variables of 
                         ! prescribe we receive temporary variables back this 
                         ! helps us to calculate the weighted mean of the newly established 
                         ! vegetation and the vegetation that is already there. For some 
                         ! other variables we receive dummies as we are simply not
                         ! going to use these values but will continue using the values of the
                         ! vegetation that is already there.

                         ! Debug
                         IF(printlev_loc>=4)THEN
                            WRITE(numout,*) 'Merging our biomass to the youngest age class.'
                            WRITE(numout,*) 'Biomass already in the youngest age class.'
                            WRITE(numout,*) 'ipts,iyoung,ivm: ',ipts,iyoung,ivm
                            WRITE(numout,*) 'share_young: ',share_young
                            WRITE(numout,*) 'veget_cov_max(young and current): ',&
                                 veget_cov_max(ipts,iyoung),veget_cov_max(ipts,ivm)
                         ENDIF
                         !-

                         !! 5.2.3.1 Initialize new and dummy variables
                         new_circ_class_biomass(:,:,:,:,:) = zero
                         new_circ_class_n(:,:,:) = zero
                         new_lm_lastyearmax(:,:) = zero
                         new_age(:,:) = zero
                         new_leaf_frac(:,:,:) = zero
                         new_atm_to_bm(:,:,:) = zero
                         new_everywhere(:,:) = zero 
                         dum_when_growthinit(:,:) = large_value
                         dum_npp_longterm(:,:) = zero
                         dum_PFTpresent(:,:) = .TRUE.
                         dum_plant_status(:,:) = iprescribe
                
                         !! 5.2.3.2 Merge the new and already available vegetation

                         ! Assign value to moiavail_growingseason
                         ! We do this differently from LCC, because we are replanting
                         ! the same stand that we just cut.
                         moiavail_growingseason(ipts,iyoung) = &
                              moiavail_growingseason(ipts,iyoung) * &
                              share_young + moiavail_growingseason(ipts,ivm) * &
                              (un - share_young)
                         wstress_season(ipts,iyoung) = wstress_season(ipts,iyoung) * &
                              share_young + wstress_season(ipts,ivm) * (un - share_young)
                         nstress_season(ipts,iyoung) = nstress_season(ipts,iyoung) * &
                              share_young + nstress_season(ipts,ivm) * (un - share_young)

     
                         !! 5.2.3.3 Initialize the newly established vegetation: 
                         !  Initialize the newly established vegetation: density of 
                         !  individuals, biomass, allocation factors, leaf age distribution, 
                         !  etc to some reasonable value ::veget_cov_max is not updated yet, 
                         !  therefore loss_gain is passed in the argument list.
                         !  For loss_gain, we only want to replant the current PFT.
                         !  If lpft_replat is TRUE we should never be here in the 
                         !  first place but to be safe lpft_replant is passed into
                         !  prescribe. This is an optional variable that will prevent
                         !  prescribing biomass if species change is used.

                         ! Debug
                         IF(printlev_loc>=4)THEN
                            IF(lpft_replant(ipts,ivm))THEN
                               WRITE(numout,*) 'ERROR - mortality_clean should never be here'
                               IF (err_act.GT.1) CALL ipslerr_p(3, &
                                    'mortality_clean in stomate_kill.f90',&
                                    'species change was activated, replanting needed',&
                                    'do not replant in mortality_clean, wait for lcchange','')
                            ENDIF
                         ENDIF
                         !-

                         loss_gain(:,:)=zero
                         loss_gain(ipts,ivm)=veget_cov_max(ipts,ivm)
                         ! In gap_clean stomate_prescribe is only used to establish new
                         ! vegetation. The variables required for recruitment are not 
                         ! passed all the way into this routine but are made dummy
                         ! variables. A more elegant solution is to use OPTIONAL variables
                         ! but the code became a bit confused because lpft_replant is
                         ! already an OPTINAL variable but for a completly different 
                         ! functionality.
                         ! Dummy for light_tran_tot_season
                         dummy1(:,:,:) = un
                         ! Dummy for EndOfYear
                         dummy2 = .FALSE.
                         CALL prescribe (npts,loss_gain, dt_days, &
                              dum_PFTpresent, new_everywhere, dum_when_growthinit, &
                              new_leaf_frac, circ_class_dist, new_circ_class_n, &
                              new_circ_class_biomass, new_atm_to_bm, forest_managed, &
                              KF, dum_plant_status, new_age, dum_npp_longterm, &
                              new_lm_lastyearmax, tau_eff_leaf, tau_eff_sap, &
                              tau_eff_root, k_latosa_adapt, dummy1, &
                              dummy2, lpft_replant,species_change_map)


                         !! 5.2.3.3.1 Merge biomass of new and already available trees
                         ! Unlike LCC, we are only at this point if we are dealing
                         ! with forests, so we don't have to check for that here.

                         ! Copy the number of individuals and the biomass of the established 
                         ! vegetation to a temporary variable. In the subroutine ::circ_class_n
                         ! and ::circ_class_biomass will be overwritten with the characteristics
                         ! of the merged vegetation
                         ! The term "established" here refers to the youngest age class, while
                         ! "tmp" is the prescribed vegetation that we just created.
                         est_circ_class_n(:) = circ_class_n(ipts,iyoung,:)
                         est_circ_class_biomass(:,:,:) = circ_class_biomass(ipts,iyoung,:,:,:)
                         tmp_circ_class_n(:) = new_circ_class_n(ipts,ivm,:)
                         tmp_circ_class_biomass(:,:,:) = new_circ_class_biomass(ipts,ivm,:,:,:)
                         
                         ! Store circumference (m) in a temporary variable that can be 
                         ! sorted from small to large - note that the dimension of these
                         ! variables are 2*ncirc
                         est_circ_class_dia(:) = &
                              wood_to_dia(est_circ_class_biomass(:,:,icarbon),ivm)
                         tmp_circ_class_dia(:) = &
                              wood_to_dia(tmp_circ_class_biomass(:,:,icarbon),ivm)

                         ! Debug
                         IF(printlev_loc>=4)THEN
                            WRITE(numout,*) 'stomate_kill share_young, ',ipts, ivm, share_young
                            WRITE(numout,*) 'rest, ', SUM(circ_class_n(ipts,ivm,:)), &
                              SUM(est_circ_class_n), SUM(tmp_circ_class_n), &
                              SUM(SUM(circ_class_biomass(ipts,ivm,:,:,icarbon),2),1), &
                              SUM(est_circ_class_biomass), &
                              SUM(tmp_circ_class_biomass), &
                              ipts, iyoung, SUM(est_circ_class_dia), SUM(tmp_circ_class_dia), &
                              lpft_replant(ipts,ivm)
                         ENDIF
                         !-

                         ! Merge the biomass 
                         CALL merge_biomass_pfts(npts, share_young, circ_class_n, &
                              est_circ_class_n, tmp_circ_class_n, circ_class_biomass, &
                              est_circ_class_biomass, &
                              tmp_circ_class_biomass, circ_class_dist, &
                              ipts, iyoung, est_circ_class_dia, tmp_circ_class_dia)

                         !! 5.2.3.4 Calculate the PFT characteristics of the merged PFT
                         !  Take the weighted mean of the existing vegetation and the new 
                         !  vegetation in this PFT. Note that co2_to_bm is in gC. m-2 dt-1 
                         !  so we should also take the weighted mean (rather than sum if
                         !  this where absolute values).
                         lm_lastyearmax(ipts,iyoung) = share_young * &
                              lm_lastyearmax(ipts,iyoung) + &
                              (un - share_young) * new_lm_lastyearmax(ipts,ivm)
                         age(ipts,iyoung) = share_young * age(ipts,iyoung) + &
                              (un - share_young) * new_age(ipts,ivm)
                         leaf_frac(ipts,iyoung,:) = share_young * leaf_frac(ipts,iyoung,:) + &
                              (un - share_young) * new_leaf_frac(ipts,ivm,:)

                         ! Note that the purpose of the prescribe subroutine is
                         ! to get biomass (seedlings) without having to simulate
                         ! seed germination and growth of very small plants.
                         ! This means that through prescribe we get biomass
                         ! without GPP, Ra and Rg. They are accounted for for
                         ! consistency reasons. Germination and growth of
                         ! seedlings is by-passed through atm_to_bm so this
                         ! really needs to ba accounted for.
                         atm_to_bm(ipts,iyoung,:) = share_young * atm_to_bm(ipts,iyoung,:) + &
                              (un - share_young) * atm_to_bm(ipts,ivm,:)
                         gpp_daily(ipts,iyoung) = share_young * gpp_daily(ipts,iyoung) + &
                              (un - share_young) * gpp_daily(ipts,ivm)
                         resp_maint(ipts,iyoung) = share_young * resp_maint(ipts,iyoung) + &
                              (un - share_young) * resp_maint(ipts,ivm)
                         resp_growth(ipts,iyoung) = share_young * resp_growth(ipts,iyoung) + &
                              (un - share_young) * resp_growth(ipts,ivm) 

                         ! Everywhere deals with the migration of vegetation. Copy the
                         ! status of the most migrated vegetation for the whole PFT
                         everywhere(ipts,iyoung) = MAX(everywhere(ipts,iyoung), &
                              new_everywhere(ipts,ivm))

                         ! The new soil&litter pools are the weighted mean of the newly 
                         ! established vegetation for that PFT and the vegetation that 
                         ! already exists in that PFT.  Notice that we do not use the
                         ! "bank" concept present in LCC because we are replanting
                         ! the same PFT which already existed, just in a younger class.
                         litter(ipts,:,iyoung,:,:) = share_young * litter(ipts,:,iyoung,:,:) + &
                              (un - share_young) * litter(ipts,:,ivm,:,:)
                         som(ipts,:,iyoung,:) =  share_young * som(ipts,:,iyoung,:) + &
                              (un - share_young) * som(ipts,:,ivm,:)
                         DO ilev=1,nlevs
                            lignin_struc(ipts,iyoung,ilev) = &
                                 litter_weight_young(istructural,ilev) * &
                                 lignin_struc(ipts,iyoung,ilev) + &
                                 (un - litter_weight_young(istructural,ilev)) * &
                                 lignin_struc(ipts,ivm,ilev) 
                            lignin_wood(ipts,iyoung,ilev) = &
                                 litter_weight_young(iwoody,ilev) * &
                                 lignin_wood(ipts,iyoung,ilev) + &
                                 (un - litter_weight_young(iwoody,ilev) ) * &
                                 lignin_wood(ipts,ivm,ilev) 
                         ENDDO
                         bm_to_litter(ipts,iyoung,:,:) = share_young * &
                              bm_to_litter(ipts,iyoung,:,:) + & 
                              (un - share_young) * bm_to_litter(ipts,ivm,:,:)

                         ! Update the soil pools
                         soil_n_min(ipts,iyoung,:) = &
                              share_young * soil_n_min(ipts,iyoung,:) + &
                              soil_n_min(ipts,ivm,:) * (un - share_young)
                         p_O2(ipts,iyoung) = share_young * p_O2(ipts,iyoung) + &
                              p_O2(ipts,ivm) * (un - share_young)
                         bact(ipts,iyoung) = share_young * bact(ipts,iyoung) + &
                              bact(ipts,ivm) * (un - share_young)

                         ! Update plant N uptake
                         n_uptake_daily(ipts,iyoung,iammonium) = share_young * &
                              n_uptake_daily(ipts,iyoung,iammonium) + &
                              n_uptake_daily(ipts,ivm,iammonium) * (un - share_young)
                         n_uptake_daily(ipts,iyoung,initrate) = share_young * &
                              n_uptake_daily(ipts,iyoung,initrate) + &
                              n_uptake_daily(ipts,ivm,initrate) * (un - share_young)

                         ! Leaf properties
                         cn_leaf_min_season(ipts,iyoung) = &
                              share_young * cn_leaf_min_season(ipts,iyoung) + &
                              cn_leaf_min_season(ipts,ivm) * (un - share_young)

                         ! For the analytic spinup, we need longterm CN ratios
                         ! calculated over the spinup period. If the tree grows
                         ! and move to an older age class, this is taken care of
                         ! by age_class_distr, but if PFT is killed we need to
                         ! move CN back to the first age class as well.
                         ! Otherwise, we will not get a CN ratio for soil and
                         ! litter pools the corresponds to the spinup period. 
                         IF(spinup_analytic)THEN
                            CN_som_litter_longterm(ipts,iyoung,:)=&
                              share_young * CN_som_litter_longterm(ipts,iyoung,:)+ &
                              CN_som_litter_longterm(ipts,ivm,:) * (un -share_young)
                            matrixA(ipts,iyoung,:,:)=&
                              share_young *matrixA(ipts,iyoung,:,:) + &
                              matrixA(ipts,ivm,:,:) * (un -share_young)
                            matrixV(ipts,iyoung,:,:)=&
                              share_young *matrixV(ipts,iyoung,:,:) + &
                              matrixV(ipts,ivm,:,:) * (un -share_young)
                            vectorB(ipts,iyoung,:)=&
                              share_young *vectorB(ipts,iyoung,:) + &
                              vectorB(ipts,ivm,:) * (un -share_young)
                            vectorU(ipts,iyoung,:)=&
                              share_young *vectorB(ipts,iyoung,:) + &
                              vectorB(ipts,ivm,:) * (un -share_young)

                         ENDIF

                         ! Now move the veget_cov_max from the current class to the young one.
                         veget_cov_max(ipts,iyoung)=veget_cov_max(ipts,iyoung)+veget_cov_max(ipts,ivm)

                      ENDIF
                
                      ! Reset all these variables, since the PFT is dead.
                      PFTpresent(ipts,ivm) = .FALSE.
                      veget_cov_max(ipts,ivm) = zero
                      lm_lastyearmax(ipts,ivm) = zero
                      age(ipts,ivm) = zero
                      leaf_frac(ipts,ivm,:) = zero
                      atm_to_bm(ipts,ivm,:) = zero
                      gpp_daily(ipts,ivm) = zero
                      resp_maint(ipts,ivm) = zero
                      resp_growth(ipts,ivm) = zero
                      everywhere(ipts,ivm) = zero
                      litter(ipts,:,ivm,:,:) = zero
                      som(ipts,:,ivm,:) = zero
                      lignin_struc(ipts,ivm,:) = zero
                      lignin_wood(ipts,ivm,:) = zero
                      bm_to_litter(ipts,ivm,:,:) = zero
                      soil_n_min(ipts,ivm,:) = zero
                      cn_leaf_min_season(ipts,ivm) = zero
                      bact(ipts,ivm) = zero
                      p_O2(ipts,ivm) = zero
                      nstress_season(ipts,ivm) = zero
                      CN_som_litter_longterm(ipts,ivm,:)=zero 
                      matrixA(ipts,ivm,:,:) = zero
                      matrixV(ipts,ivm,:,:) = zero
                      vectorB(ipts,ivm,:) =  zero
                      vectorU(ipts,ivm,:) = zero
                      
                                           
                      ! Keep ::biomass and circ_class_biomass synchronized
                      circ_class_biomass(ipts,ivm,:,:,:) = zero
                      circ_class_n(ipts,ivm,:) = zero
                      
                   ENDIF ! If this is the youngest age class

                ENDIF ! If we are using age classes

             ENDIF ! All biomass in ivm and plant the same species

          ENDIF ! is_tree

       ENDDO  ! loop over pfts

    END DO ! loop over land points

 !! 3. Check numerical consistency of this routine

    IF (err_act.GT.1) THEN

       ! 3.2 Check surface area
       CALL check_surface_area("mortality_clean", npts, veget_cov_max_begin, &
            veget_cov_max,'ageclass')

       ! 3.3 Mass balance closure 
       ! 3.3.1 Calculate final biomass
       pool_end(:,:,:) = zero 
       DO iele = 1,nelements
          DO ipar = 1,nparts
             DO icir = 1,ncirc
                pool_end(:,:,iele) = pool_end(:,:,iele) + &
                     (circ_class_biomass(:,:,icir,ipar,iele) * &
                     circ_class_n(:,:,icir) * veget_cov_max(:,:)) 
             ENDDO
          ENDDO

          ! Litter pool (gC m-2) *  (m2 m-2) 
          DO ilit = 1,nlitt
             DO ilev = 1,nlevs
                pool_end(:,:,iele) = pool_end(:,:,iele) + &
                     litter(:,ilit,:,ilev,iele) * veget_cov_max(:,:)
             ENDDO
          ENDDO

          ! Soil carbon (gC m-2) *  (m2 m-2)
          DO icarb = 1,ncarb
             pool_end(:,:,iele) = pool_end(:,:,iele) + &
                  som(:,icarb,:,iele) * veget_cov_max(:,:)
          ENDDO

       ENDDO

       ! Denitrifying bacteria (only C)
       pool_end(:,:,icarbon) = pool_end(:,:,icarbon) + &
            bact(:,:) * veget_cov_max(:,:)
       
       ! Nitrogen only
       DO ispec = 1,nnspec
          pool_end(:,:,initrogen) = pool_end(:,:,initrogen) + &
               soil_n_min(:,:,ispec) * veget_cov_max(:,:)
       ENDDO
    
       ! 3.3.2 Calculate mass balance
       ! Common processes
       DO iele=1,nelements
          check_intern(:,:,iatm2land,iele) = &
               check_intern(:,:,iatm2land,iele) + &
               atm_to_bm(:,:,iele) * veget_cov_max(:,:) 
          check_intern(:,:,ipoolchange,iele) = -un * (pool_end(:,:,iele) - &
               pool_start(:,:,iele)) 
       ENDDO
   
       ! Specific fluxes
       check_intern(:,:,iatm2land,icarbon) = &
            check_intern(:,:,iatm2land,icarbon) + &
            gpp_daily(:,:) * veget_cov_max(:,:)
       check_intern(:,:,iland2atm,icarbon) = &
            check_intern(:,:,iland2atm,icarbon) -un * &
            (resp_maint(:,:) + resp_growth(:,:)) * &
            veget_cov_max(:,:)

       !! 1.3 Initialize check for area conservation
       veget_cov_max_begin(:,:) = veget_cov_max(:,:)
       closure_intern(:,:,:) = zero
       DO imbc = 1,nmbcomp
          DO iele=1,nelements
             ! Debug
             IF (printlev_loc>=4) WRITE(numout,*) &
                  'check_intern, ivm, imbc, iele, ', imbc, &
                  iele, check_intern(:,test_pft,imbc,iele)
             !-
             closure_intern(:,:,iele) = closure_intern(:,:,iele) + &
                  check_intern(:,:,imbc,iele)
          ENDDO
       ENDDO
       
       ! 4.3.3 Check mass balance closure
       CALL check_mass_balance("mortality_clean", closure_intern, npts, &
            pool_end, pool_start, veget_cov_max, 'ageclass')
       
    ENDIF ! err_act.GT.1
    
    IF (printlev.GE.4) WRITE(numout,*) 'Leaving mortality_clean'
    
  END SUBROUTINE mortality_clean

END MODULE stomate_kill
